%
% nethistBSRate(dname,laps,Ne_C3,cells,wMax)
%
% Generate figure demonstrating BS
%

function nethistBSRate(dname0,laps,Ne_C3,cells,wMax,handles)
% cells: cells to be plotted
% wMax: max EtoE weight

persistent dname

if ~isempty(dname0)
    dname = dname0;
end

cd(dname)

if ~isempty(handles)
    guiFlag = 1;
else
    guiFlag = 0;
end

s = 1;

% % load WEtoE, row = pre, col = post
% WE = load('WEtoE-fin-ses-1');
% WEtoE = spalloc(Ne_C3,Ne_C3,ceil(.3*Ne_C3^2));  %final weight matrix
% for k=1:size(WE,1)
%     WEtoE(WE(k,1),WE(k,2)) = WE(k,3);
% end

beta = 10e-3;  %kHz
alpha = 100;  %ms

dspkVec = 50*(14:-1:0)';  %ms
expVec = exp(-dspkVec/alpha);

if ~isempty(cells)
    Ncells = length(cells);
    rate = zeros(1,360);

    bar1 = zeros(1,360);
    bar2 = zeros(1,360);
    Rcell = zeros(laps,360);

    for k = 1:Ncells
        cell = cells(k);

        bar1 = 0*bar1;
        bar2 = 0*bar2;
        Rcell = 0*Rcell;

        for ell = 1:laps

            % get CA3-E spike histogram
            str = [num2str(s) '-lap-' num2str(ell)];
            stre = ['ca3e-data-ses-' str];
            a = fopen(stre,'r');
            dat = fread(a,'int16');
            fclose(a);
            ca3ehist = ghist(dat,Ne_C3);  %Ne_C3 x 360

            rate = 0*rate;  %spike rate for each lap
            for deg = 1:360
                if deg <= 14
                    if ell == 1
                        rate(deg) = beta*ca3ehist(cell,1:deg)*expVec(end-deg+1:end);
                    else
                        %                     rate(deg) = beta*ca3ehist(cell,[(346+deg):360,1:deg])*expVec;
                        spks = [ca3ehistLast(cell,(346+deg):360), ca3ehist(cell,1:deg)];
                        rate(deg) = beta*spks*expVec;
                    end
                else
                    rate(deg) = beta*ca3ehist(cell,(deg-14):deg)*expVec;
                end
            end  %for deg

            Rcell(ell,:) = rate;

            if 3<=ell && ell <=7
                bar1 = bar1+rate;
                if ell == 7
                    Rcell1 = bar1/5;
                    [place1,flag1]=findPlace(Rcell1);
                end
            elseif ell >= laps-4
                bar2 = bar2+rate;
                if ell == laps
                    Rcell2 = bar2/5;
                    [place2,flag2]=findPlace(Rcell2);
                end

            end

            ca3ehistLast = ca3ehist;

        end  %for laps

        if flag1 == 0 || flag2 == 0
            fprintf('cell: %i, firing rate too low\n', cell)
        elseif flag1 == -1 || flag2 == -1
            fprintf('cell: %i, multiple firint fields\n', cell)
        elseif place1 - place2 > 1
            fprintf('cell: %i, p1: %4.1f, p2: %4.1f, BS: %4.1f\n',cell,place1,place2,place2-place1)
        elseif place1-place2 < -1
            fprintf('cell: %i, FS: %i\n', cell,place2-place1)
        else
            fprintf('cell: %i, NS: %i\n', cell,place2-place1)
        end


        bar1ave = bar1/5;
        bar2ave = bar2/5;

        figure(2*(k-1)+1)
        subplot(1,2,1)
        imagesc(1e3*Rcell')
        xlabel('Laps','fontsize',18)
        ylabel('Angle (deg) ','fontsize',18)
        title(['Spike Rate per Lap, cell ' num2str(cell) ' '],'fontsize',18)
        colorbar

        subplot(1,2,2)
        syn = find(WEtoE(:,cell));
        WEtoEsyn = WEtoE(syn,cell);

        cent = linspace(0,wMax,20);
        [numofW,Wvals] = hist(WEtoEsyn,cent);
        bar(Wvals,numofW);
        axis([0 wMax 0 length(syn)])

        xlabel('Weight','fontsize',18)
        ylabel('Number of Cells','fontsize',18)
        title('Final Weight Distribution','fontsize',18)

        figure(2*k)
        subplot(2,1,1)
        bar(1:360,bar1ave*1e3,'r')
        title(['Laps 3-7, cell ' num2str(cell)],'fontsize',18)
        xlim([265 315])
        
        subplot(2,1,2)
        bar(1:360,bar2ave*1e3,'b')
        xlabel('Angle (deg) ','fontsize',18)
        ylabel('Spike Rate (Hz) ','fontsize',18)
        title(['Laps ' num2str(laps-4) '-' num2str(laps) ', cell ' num2str(cell) ' '],'fontsize',18)
        xlim([265 315])
        
    end  %for cells

else
% See what happens across entire run

rate = zeros(Ne_C3,360);
bar1 = rate;
bar2 = rate;

chgCells = spalloc(1,Ne_C3,ceil(.1*Ne_C3));

for ell = [2:7, laps-5:laps]

    % get CA3-E spike histogram
    str = [num2str(s) '-lap-' num2str(ell)];
    stre = ['ca3e-data-ses-' str];
    a = fopen(stre,'r');
    dat = fread(a,'int16');
    fclose(a);
    ca3ehist = ghist(dat,Ne_C3);  %Ne_C3 x 360

    rate = 0*rate;  %spike rate for each lap
    if ell == 2 || ell == laps-5
        degVec = 300:360;
    else
        degVec = 1:360;
    end
    
    for deg = degVec
        if deg <= 14
            if ell == 1
                rate(:,deg) = beta*ca3ehist(:,1:deg)*expVec(end-deg+1:end);
            else
                spks = [ca3ehistLast(:,(346+deg):360), ca3ehist(:,1:deg)];
                rate(:,deg) = beta*spks*expVec;
            end
        else
            rate(:,deg) = beta*ca3ehist(:,(deg-14):deg)*expVec;
        end
    end  %for deg

    if ell == 3
        rate1 = rate;
    elseif ell == 4
        rate2 = rate;
    elseif ell == 5
        rate3 = rate;
        rateA = (rate1+rate2+rate3)/3;
        [pOld,fOld]=findPlace(rateA);
        rate1 = rate2;
        rate2 = rate3;
    end
    if ell >= 6
        rate3 = rate;
        rateA = (rate1+rate2+rate3)/3;
        [pNew,fNew] = findPlace(rateA);
        chg = find(abs(pNew-pOld) > 40);
        chgCells(chg) = 1;
        rate1 = rate2;
        rate2 = rate3;
    end
            
    
    if 3<=ell && ell <=7
        bar1 = bar1+rate;
        if ell == 7
            R1 = bar1/5;
            [places1,flags1] = findPlace(R1);
        end

    elseif ell >= laps-4
        bar2 = bar2+rate;
        if ell == laps
            R2 = bar2/5;
            [places2,flags2] = findPlace(R2);
        end
    end

    ca3ehistLast = ca3ehist;
    
end  %for laps

p1 = places1;
p2 = places2;
for k=1:Ne_C3
    p1k = p1(k);
    p2k = p2(k);
    if p1k<10 && p2k>350
        p1(k) = p1k+360;
    elseif p1k>350 && p2k<10
        p2(k) = p2k+360;
    end
end

d = p2-p1;

big = find(abs(d)>20);

flags2(big) = -2;  %change firing field

flags = flags1+flags2;
ind = find(flags==2);  %place cells

dgood = d(ind);

totalShift = sum(dgood);
meanShift = mean(dgood);

BS = ind(find(dgood<=-1));  %backward shift
FS = ind(find(dgood>=1));  %forward shift
NS = ind(intersect(find(dgood>-1), find(dgood<1)));  %no shift

Nplace = length(BS)+length(FS)+length(NS);

bad = setdiff(1:Ne_C3,ind);
Nbad = length(bad);

NF = zeros(1,Nbad);
MF = zeros(1,Nbad);
CF = zeros(1,Nbad);

cn = 0;
cm = 0;
cc = 0;

for k=1:length(bad)
    bc = bad(k);
    if flags1(bc) == 0 || flags2(bc) == 0
        cn = cn+1;
        NF(cn) = bc;
    elseif flags1(bc) == -1 || flags2(bc) == -1
        cm = cm + 1;
        MF(cm) = bc;
    elseif flags2(bc) == -2
        cc = cc + 1;
        CF(cc) = bc;
    else
        disp('unknown flag')
        keyboard
    end
end

NF = NF(1:cn);
MF = MF(1:cm);
CF = CF(1:cc);

BSP = 100*length(BS)/Nplace;
FSP = 100*length(FS)/Nplace;
NSP = 100*length(NS)/Nplace;

NFP = 100*length(NF)/Nplace;
MFP = 100*length(MF)/Nplace;
CFP = 100*length(CF)/Nplace;

if guiFlag == 0
    fprintf('BS: %3.2f%%, FS: %3.2f%%, NS: %3.2f%%\n',BSP,FSP,NSP)
    fprintf('NF: %3.2f%%, MF: %3.2f%%, CF: %3.2f%%\n', NFP,MFP,CFP)
    fprintf('Total shift: %i, Mean Shift: %i\n', totalShift,meanShift)
else
    axes(handles.axes_bot)
    bar(1:3,[BSP,FSP,NSP])
    ylabel('Percentage of Cells','fontsize',18)
    title('Shift of Cells','fontsize',18)
    set(gca,'XTickLabel',{'Backward Shift','Forward Shift','No Shift'},'fontsize',16)
    cd ..
    return
end

cent = -20:.5:20;
N = hist(dgood,cent);
figure
bar(cent,100*N/Nplace);
hold on
plot([0 0], [0 100],'--r')
axis([-20 20 0 max(100*N/Nplace+2)])
hold off

xlabel('Shift','fontsize',18)
ylabel('Percentage of Place Cells','fontsize',18)
title('Histogram of Place Field Shift','fontsize',18)

end

cd ..
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function shist = ghist(dat,dim)

shist = zeros(dim,360);
rind = find(dat < -0.5);
rind(end+1) = rind(end)+length(dat(rind(end):end));
st = 2;

for j=1:360,

    fin = rind(j+1)-1;
    cind = dat(st:fin);
    shist(:,j) = 0;

    for k=1:length(cind)
        shist(cind(k),j) = shist(cind(k),j) + 1;
    end

    st = rind(j+1)+1;


end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [place, flag]= findPlace(R)
%
% input: Rcell: average spike rate of the cell (Hz)  (1 x 360)
% output: place: place field of the cell; 0 means it fails to meet the
% place field criteria
%         flag: 1: have a place cell
%               0: firing rate is too low
%               -1: multiple firing rates
%
%Criteria: place = location of max firing rate
%          max firing rate must be at least 3 Hz
%          outside of place +- 10 degrees, cannot have a rate greater than
%          1/3 its max rate
%

%R = Ncellsx360

[maxR,Mvec] = max(R,[],2);  %Mvec = degree of maximal spiking for each cell, 1 x Ncells
Ncells = size(R,1);

Rtog = [R(:,341:360) R R(:,1:20)];  %1 x 400

tol = 1e-5;

place = zeros(1,Ncells);
flag = place;
for cell = 1:Ncells
    if maxR(cell) >= 3e-3  %strong firing field (larger than 3 Hz)
 
        Rcell = R(cell,:);
        
        M = Mvec(cell);
        Mtog = M + 20;

        ind2 = find(Rtog(cell,Mtog+1:end)<tol,1)+M;
        ind1 = find(Rtog(cell,1:Mtog-1)<tol,1,'last')-20;
        
        if ind2 > 360
            ind3 = mod(ind2,360);
        elseif ind1<=0
            ind0 = 360+ind1;
        end

        rem = setdiff(1:360,ind1:ind2);
        if any(Rcell(rem) > maxR(cell)/3)  %multiple firing fields
            flag(cell) = -1;
        else  %single firing field larger than 3 Hz
%             place(cell) = M;
         
            if ind2>360
                degs = [ind1:360 1:ind3];
            elseif ind1 <= 0
                degs = [ind0:360 1:ind2];
            else
                degs = ind1:ind2;
            end
            
            Mass = sum(Rcell(degs));
            s = 0;
            
            for deg = degs
                s = s + deg*Rcell(deg);
            end

            com = s/Mass;
            place(cell) = com;
            
            flag(cell) = 1;
        end
    end
end



